import { TimezonesResult, accessoryAttributes, apiClientManager, mergeRequestOptions, taskCatalog, } from "@trigger.dev/core/v3";
import { zodfetch } from "@trigger.dev/core/v3/zodfetch";
import { createTask } from "../shared.js";
import { tracer } from "../tracer.js";
export function task(params) {
    const task = createTask(params);
    const cron = params.cron
        ? typeof params.cron === "string"
            ? params.cron
            : params.cron.pattern
        : undefined;
    const timezone = (params.cron && typeof params.cron !== "string" ? params.cron.timezone : "UTC") ?? "UTC";
    taskCatalog.updateTaskMetadata(task.id, {
        triggerSource: "schedule",
        schedule: cron
            ? {
                cron: cron,
                timezone,
            }
            : undefined,
    });
    return task;
}
/**
 * Creates a new schedule
 * @param options
 * @param options.task - The identifier of the task to be scheduled (Must already exist and be a scheduled task)
 * @param options.cron - The cron expression for the schedule (e.g. `0 0 * * *`)
 * @param options.timezone - An optional timezone for the schedule in the IANA format (e.g. `America/Los_Angeles`). Defaults to "UTC".
 * @param options.externalId - An optional external identifier for the schedule
 * @param options.deduplicationKey - An optional deduplication key for the schedule
 * @returns The created schedule
 */
export function create(options, requestOptions) {
    const apiClient = apiClientManager.clientOrThrow();
    const $requestOptions = mergeRequestOptions({
        tracer,
        name: "schedules.create()",
        icon: "clock",
        attributes: {
            ...accessoryAttributes({
                items: [
                    {
                        text: options.cron,
                        variant: "normal",
                    },
                ],
                style: "codepath",
            }),
        },
    }, requestOptions);
    return apiClient.createSchedule(options, $requestOptions);
}
/**
 * Retrieves a schedule
 * @param scheduleId - The ID of the schedule to retrieve
 * @returns The retrieved schedule
 */
export function retrieve(scheduleId, requestOptions) {
    const apiClient = apiClientManager.clientOrThrow();
    const $requestOptions = mergeRequestOptions({
        tracer,
        name: "schedules.retrieve()",
        icon: "clock",
        attributes: {
            scheduleId,
            ...accessoryAttributes({
                items: [
                    {
                        text: scheduleId,
                        variant: "normal",
                    },
                ],
                style: "codepath",
            }),
        },
    }, requestOptions);
    return apiClient.retrieveSchedule(scheduleId, $requestOptions);
}
/**
 * Updates a schedule
 * @param scheduleId - The ID of the schedule to update
 * @param options - The updated schedule options
 * @param options.task - The identifier of the task to be scheduled (Must already exist and be a scheduled task)
 * @param options.cron - The cron expression for the schedule (e.g. `0 0 * * *`)
 * @param options.timezone - An optional timezone for the schedule in the IANA format (e.g. `America/Los_Angeles`). Defaults to "UTC".
 * @param options.externalId - An optional external identifier for the schedule
 * @returns The updated schedule
 */
export function update(scheduleId, options, requestOptions) {
    const apiClient = apiClientManager.clientOrThrow();
    const $requestOptions = mergeRequestOptions({
        tracer,
        name: "schedules.update()",
        icon: "clock",
        attributes: {
            scheduleId,
            ...accessoryAttributes({
                items: [
                    {
                        text: scheduleId,
                        variant: "normal",
                    },
                ],
                style: "codepath",
            }),
        },
    }, requestOptions);
    return apiClient.updateSchedule(scheduleId, options, $requestOptions);
}
/**
 * Deletes a schedule
 * @param scheduleId - The ID of the schedule to delete
 */
export function del(scheduleId, requestOptions) {
    const apiClient = apiClientManager.clientOrThrow();
    const $requestOptions = mergeRequestOptions({
        tracer,
        name: "schedules.delete()",
        icon: "clock",
        attributes: {
            scheduleId,
            ...accessoryAttributes({
                items: [
                    {
                        text: scheduleId,
                        variant: "normal",
                    },
                ],
                style: "codepath",
            }),
        },
    }, requestOptions);
    return apiClient.deleteSchedule(scheduleId, $requestOptions);
}
/**
 * Deactivates a schedule
 * @param scheduleId - The ID of the schedule to deactivate
 */
export function deactivate(scheduleId, requestOptions) {
    const apiClient = apiClientManager.clientOrThrow();
    const $requestOptions = mergeRequestOptions({
        tracer,
        name: "schedules.deactivate()",
        icon: "clock",
        attributes: {
            scheduleId,
            ...accessoryAttributes({
                items: [
                    {
                        text: scheduleId,
                        variant: "normal",
                    },
                ],
                style: "codepath",
            }),
        },
    }, requestOptions);
    return apiClient.deactivateSchedule(scheduleId, $requestOptions);
}
/**
 * Activates a schedule
 * @param scheduleId - The ID of the schedule to activate
 */
export function activate(scheduleId, requestOptions) {
    const apiClient = apiClientManager.clientOrThrow();
    const $requestOptions = mergeRequestOptions({
        tracer,
        name: "schedules.activate()",
        icon: "clock",
        attributes: {
            scheduleId,
            ...accessoryAttributes({
                items: [
                    {
                        text: scheduleId,
                        variant: "normal",
                    },
                ],
                style: "codepath",
            }),
        },
    }, requestOptions);
    return apiClient.activateSchedule(scheduleId, $requestOptions);
}
/**
 * Lists schedules
 * @param options - The list options
 * @param options.page - The page number
 * @param options.perPage - The number of schedules per page
 * @returns The list of schedules
 */
export function list(options, requestOptions) {
    const apiClient = apiClientManager.clientOrThrow();
    const $requestOptions = mergeRequestOptions({
        tracer,
        name: "schedules.list()",
        icon: "clock",
    }, requestOptions);
    return apiClient.listSchedules(options, $requestOptions);
}
/**
 * Lists the possible timezones we support
 * @param excludeUtc - By default "UTC" is included and is first. If true, "UTC" will be excluded.
 */
export function timezones(options) {
    const baseUrl = apiClientManager.baseURL;
    return zodfetch(TimezonesResult, `${baseUrl}/api/v1/timezones${options?.excludeUtc === true ? "?excludeUtc=true" : ""}`, {
        method: "GET",
        headers: {
            "Content-Type": "application/json",
        },
    });
}
//# sourceMappingURL=index.js.map